/* include file for recon.c  and recon_sub.c */

/* recon -- performs grayscale reconstruction of an image from a smoothed */
/*            version.                                                      */
/*                                                                          */
/* This is an implementation of the fast hybrid grayscale reconstruction    */
/* algorithm described in Vincent, L., "Morphological reconstruction in     */
/* image analysis: applications and efficient algorithms, IEEE Trans. Image */
/* Processing, Vol. 2, No. 2, April 1993.                                   */
/*                                                                          */
/* I have modified the algorithm so that (1) the smooth image need not be   */
/* everywhere less than (or greater than) the original image. (2) it uses   */
/* either an 8-connected or a 4-connected neighborhood.                     */

/* version 1.0   13 July 1993  */

/* by Richard Alan Peters II                       */
/* Department of Electrical Engineering            */
/* Vanderbilt University School of Engineering     */ 
/* Nashville, TN 37235                             */ 
/* rap2@vuse.vanderbilt.edu                        */ 
/*                                                 */ 
/* This software is freely redistributable if      */ 
/* the author's name and affiliation are included. */


#define TRUE    1
#define FALSE   0
#define BLACK   0
#define WHITE 255
#ifndef ALLOC
#define ALLOC TRUE
#endif

extern FILE *OpenFile();
extern  int  ReadRasterFile();
extern  int  WriteRasterFile();
extern void  ExtractLuminance();

void BackScanBot4();
void BackScanTop4();
void BackScanBot8();
void BackScanTop8();
void ChkTopBot();
int  Difference();
void ForScanBot4();
void ForScanTop4();
void ForScanBot8();
void ForScanTop8();
void MakeRecon();
void MakeTopBot();
void ReconBot();
void ReconSub();
void ReconTop();

#ifndef BYTE
typedef unsigned char byte;
#define BYTE 1
#endif

