/* skel_sub.c  --  contains skeleton vers. 1.0 subroutine */

/* contains: OrImg(), SetSub(), Skeleton();  */

/* dependencies: MorphSub() */

/* skeleton version 1.0   3 March 1994             */
/* morphological image noise reduction program     */
/*                                                 */
/* by Richard Alan Peters II                       */
/* Department of Electrical Engineering            */
/* Vanderbilt University School of Engineering     */
/* Nashville, TN 37235                             */
/* rap2@vuse.vanderbilt.edu                        */
/*                                                 */
/* This software is freely redistributable if      */
/* the author's name and affiliation are included. */


#include "skeleton.h"
#include "rasterio.h"

static void OrImg();
static void SetSub();



static void OrImg ( I1, I2, O, X, Y )
   byte *I1;  /* image 1 */
   byte *I2;  /* image 2 */
   byte *O;   /* result  */
   int X,Y;   /* image horizontal, vertical dimensions */
   {
   byte *A = I1;
   byte *B = I2;
   byte *C = O;
   int   n = X*Y;
   int i;

   for (i=0; i<n; ++i)
      {
      *C = (*A || *B) ? (*A ? *A : *B) : BLACK;
      ++A;
      ++B;
      ++C;
      }

   return;
   }

      

static void SetSub ( I1, I2, O, X, Y )
   byte *I1;  /* binary image 1 */
   byte *I2;  /* binary image 2 */
   byte *O;   /* binary result  */
   int X,Y;   /* image horizontal, vertical dimensions */
   {
   byte *A = I1;
   byte *B = I2;
   byte *C = O;
   int   n = X*Y;
   int i;

   for (i=0; i<n; ++i)
      {
      *C = (*A && *B) ? BLACK : *A;
      ++A;
      ++B;
      ++C;
      }

   return;
   }



void Skeleton( In, Out, X, Y, sX, sY )
   byte *In;
   byte *Out;
   int  X,Y;
   int  sX, sY;
   {
   int sx,sy;
   int stelem;
   char *SEName;
   byte *Mask1, *Mask2, *Mask3;
   byte *I, *O;
   int i,n;

   /* allocate space for image masks */

   if ( !( Mask1 = (byte *)calloc( X*Y, sizeof(byte) )) )
      {
      fprintf(stderr,"Skeleton: Unable to allocate image mask 1\n");
      exit(0);
      } 
   if ( !( Mask2 = (byte *)calloc( X*Y, sizeof(byte) )) )
      {
      fprintf(stderr,"Skeleton: Unable to allocate image mask 2\n");
      exit(0);
      } 
   if ( !( Mask3 = (byte *)calloc( X*Y, sizeof(byte) )) )
      {
      fprintf(stderr,"Skeleton: Unable to allocate image mask 3\n");
      exit(0);
      } 
 
   MorphSub( OPEN, In, Mask2, X, Y, BINIMG, FALSE, 0, 0,
             NULL, AUTO, 2, 2, 0, GRASE, SET, 0, FALSE, FALSE);

   SetSub( In, Mask2, Mask1, X ,Y );

   sx = sy = 2;

   while ( (sx < sX) && (sy < sY) )
      {
      MorphSub( ERODE, In, Mask2, X, Y, BINIMG, FALSE, 0, 0,
                NULL, AUTO, sx, sy, 0, GRASE, SET, 0, FALSE, FALSE);

      MorphSub( OPEN, Mask2, Mask3, X, Y, BINIMG, FALSE, 0, 0,
                NULL, AUTO, 2, 2, 0, GRASE, SET, 0, FALSE, FALSE);

      SetSub( Mask2, Mask3, Mask2, X, Y);

      OrImg( Mask1, Mask2, Mask1, X, Y);

      ++sx;
      ++sy;
      }

   n = X*Y;
   I = Mask1;
   O = Out;
   for ( i=0; i<n; ++i )
      *(O++) = *(I++);
   
   free(Mask1);
   free(Mask2);
   free(Mask3);

   return;
   }


