/* skeleton.c  --  skeleton vers. 1.0 main program */

/* contains: main(),  OrImg(), SetSub(), ReadIt(), WriteIt()  */

/* dependencies: Skeleton(), ReadRasterFile(), WriteRasterFile() */

/* skeleton version 1.0   3 March 1994             */
/* morphological image noise reduction program     */
/*                                                 */
/* by Richard Alan Peters II                       */
/* Department of Electrical Engineering            */
/* Vanderbilt University School of Engineering     */
/* Nashville, TN 37235                             */
/* rap2@vuse.vanderbilt.edu                        */
/*                                                 */
/* This software is freely redistributable if      */
/* the author's name and affiliation are included. */


#include "skeleton.h"
#include "rasterio.h"

#define FNLGTH    64   /* maximum file name string length */

static void ReadIt();
static void WriteIt();


static   struct rasterfile RasHd;    /* raster file header */
static   unsigned char *cmap;        /* colormap */
 
main( argc, argv ) 
   unsigned int argc;    /* count of arguments parsed by operating system    */
   char *argv[];         /* pointers to arguments parsed by operating system */
   {

   byte *Img;          /* input image    */
   byte *Sk;           /* skeleton image */
   byte *I,*S,*T;      /* image pointers */
   int X,Y;            /* image horiz, vert. dimensions  */
   int M;              /* number of pixels in image      */
   int sX,sY;          /* structuring element x,y dimensions */
   int i;              /* index variable */
   FILE *fp;           /* file pointer   */
   char fname[FNLGTH]; /* name of output file for smooth input/output */



   /* begin */



   if (argc < 3 ) /* display usage */
      {
      fprintf(stderr,
         "usage: skeleton  InFile  OutFile  x  y\n");
      exit(0);
      }



   /* initialize everything that needs it */

   X = Y = 0;
   sX = atoi( argv[3]);
   sY = atoi( argv[4]);

   /* read the input image into memory */
   ReadIt( argv[1], &RasHd, &cmap, &Img, ALLOC );


   /* get the image dimensions */

   X = RasHd.ras_width;
   Y = RasHd.ras_height;
   M = X*Y;


   /* allocate image space */

   if ( !(Sk = (byte *)calloc( M, sizeof(byte) )) ) /* smoothed image */
      {
      fprintf(stderr,"main: Unable to allocate skeleton image.\n");
      exit(0);
      }


   /* compute skeleton */

   Skeleton( Img, Sk, X, Y, sX, sY );


   /* output skeleton image */

   WriteIt( argv[2], &RasHd, cmap, Sk );



   /* free memory */

   free( Img );
   free( Sk );
   if ( cmap ) free( cmap );

   } /* end */






 static void ReadIt( name, H, C, I, Alloc )
   char *name;                /* filename */
   struct rasterfile *H;      /* raster file header */
   unsigned char **C;         /* colormap */
   byte **I;                  /* image */
   int Alloc;                 /* allocate flag */
   {
   FILE *fp;

   if ( !(fp = OpenFile( name, "IMGPATHI", "r" )) ) exit( 0 );
   ReadRasterFile( fp, H, C, I, 0, 0, 0, 0, Alloc );
   fclose( fp );

   /* extract the luminance */
   ExtractLuminance( H, *C, *I, 0, 0, 0, 0 );

   return;
   }


static void WriteIt( name, H, C, I )
   char *name;                /* filename */
   struct rasterfile H;       /* raster file header */
   unsigned char *C;          /* colormap */
   byte *I;                   /* image */
   {
   FILE *fp;

   if ( !(fp = OpenFile( name, "IMGPATHO", "w" )) ) exit( 0 );
   WriteRasterFile( fp, H, C, I, 0, 0, 0, 0 );
   fflush( fp );
   fclose( fp );

   return;
   }




