/* topbot - combine top and bot images with smooth image */
/* version 1.0     26 Oct 1992 */

/* by Richard Alan Peters II                       */
/* Department of Electrical Engineering            */
/* Vanderbilt University School of Engineering     */ 
/* Nashville, TN 37235                             */ 
/* rap2@vuse.vanderbilt.edu                        */ 
/*                                                 */ 
/* This software is freely redistributable if      */ 
/* the author's name and affiliation are included. */


#include <math.h>		
#include <stdio.h>
#include <strings.h>
#include <values.h>
#include "rasterio.h"
#include "topbot.h"

#define ALLOC 1


main( argc, argv ) 
   unsigned int argc;   /* count of arguments parsed by operating system    */
   char *argv[];        /* pointers to arguments parsed by operating system */
   {
   FILE *fp;			/* file pointer */
   struct rasterfile hB,hT,hS;	/* rasterfile headers */
   byte *cmB,*cmT,*cmS;	/* colormaps */
   byte *B,*T,*S,*O;	/* image pointers */
   byte *b,*t,*s,*o;   	/* image pointers */
   int X,Y;				/* image dimensions */
   int i,j,k;			/* indices */
   int p,bp,tp,sp;
   int n = argc;
   int absmax = FALSE;
   char *topname = NULL;
   char *botname = NULL;
   char *outname = NULL;
   char *smoname = NULL;

   if (argc < 4)
      {
      fprintf(stderr,
         "usage: topbot [-a] [-s smooth.ras] [-t top.ras] [-b bot.ras] -o out.ras\n");
      exit(0);
      }

   /* get arguments */
   while ( --n )
      {
      if ( argv[n][0] == '-' )          /* then this is a switch */
         {
         if ( argv[n][1] == 'a' )       /* absolute maximum flag */
            {
            absmax = TRUE;
            }
         else if ( argv[n][1] == 'b' )       /* bot filename follows */
            {
            botname = argv[n+1];
            if ( !(fp = OpenFile( botname, "", "r" )) ) exit( 0 );
            if (  ReadRasterFile( fp, &hB, &cmB, &B, 0, 0, 0, 0, ALLOC )  ) 
               exit( 0 );
            fclose( fp );
            }
         else if ( argv[n][1] == 'o' )       /* out filename follows */
            {
            outname = argv[n+1];
            }
         else if ( argv[n][1] == 's' )       /* smooth filename follows */
            {
            smoname = argv[n+1];
            if ( !(fp = OpenFile( smoname, "", "r" )) ) exit( 0 );
            if (  ReadRasterFile( fp, &hS, &cmS, &S, 0, 0, 0, 0, ALLOC )  ) 
               exit( 0 );
            fclose( fp );
            }
         else if ( argv[n][1] == 't' )       /* top filename follows */
            {
            topname = argv[n+1];
            if ( !(fp = OpenFile( topname, "", "r" )) ) exit( 0 );
            if (  ReadRasterFile( fp, &hT, &cmT, &T, 0, 0, 0, 0, ALLOC )  ) 
               exit( 0 );
            fclose( fp );
            }
         }
      }

   if ( !outname )
      {
      fprintf(stderr,"Must specify an output file name.\n");
      exit(0);
      }

   if ( !topname && !botname)
      {
      fprintf(stderr,"Must specify at least one of botfile and topfile.\n");
      exit(0);
      }


   /* image dimensions */
   if ( topname )
      {
      X = hT.ras_width;
      Y = hT.ras_height;
      if ( botname && ( (hB.ras_width != X)  ||  (hB.ras_height != Y) ) )
         {
         fprintf(stderr, "top and bot images have unequal dimensions\n");
         exit( 0 );
         }
      }
   else
      {
      X = hB.ras_width;
      Y = hB.ras_height;
      }
   if ( smoname && ( (hS.ras_width != X)  ||  (hS.ras_height != Y) ) )
      {
      fprintf(stderr,
        "The dimensions of the smooth file do not match the other(s).\n");
      exit( 0 );
      }

   /* make output image. */
   if ( !(O = (byte *)calloc( X*Y, sizeof(byte) )) )
         {
         fprintf(stderr, "Unable to allocate output image buffer.");
         exit( 0 );
         }


   /* main loop */
   n = X*Y;
   if ( botname && topname && smoname && !absmax )
      {
      b = B;
      t = T;
      s = S;
      o = O;
      for ( i=0; i<n; ++i)
         {
         p = *(s++) + *(t++) - *(b++);
         *(o++) = (p > WHITE) ? WHITE : ((p < BLACK) ? BLACK : p);
         }
      }
   else if ( botname && topname && smoname && absmax )
      {
      b = B;
      t = T;
      s = S;
      o = O;
      for ( i=0; i<n; ++i)
         {
         tp = *(t++);
         bp = *(b++);
         sp = *(s++);
         if ( tp > bp )
            p = sp + tp;
         else
            p = sp - bp;
         *(o++) = (p > WHITE) ? WHITE : ((p < BLACK) ? BLACK : p);
         }
      }
   else if ( smoname && botname )
      {
      b = B;
      s = S;
      o = O;
      for ( i=0; i<n; ++i)
         {
         p = *(s++) - *(b++);
         *(o++) = (p < BLACK) ? BLACK : p;
         }
      }
   else if ( smoname && topname )
      {
      t = T;
      s = S;
      o = O;
      for ( i=0; i<n; ++i)
         {
         p = *(s++) + *(t++);
         *(o++) = (p > WHITE) ? WHITE : p;
         }
      }
   else if ( botname && topname && !absmax )
      {
      b = B;
      t = T;
      o = O;
      sp = WHITE >> 1;
      for ( i=0; i<n; ++i)
         {
         p = *(t++) - *(b++) + sp;
         *(o++) = (p > WHITE) ? WHITE : ((p < BLACK) ? BLACK : p);
         }
      }
   else if ( botname && topname && absmax )
      {
      b = B;
      t = T;
      o = O;
      sp = WHITE >> 1;
      for ( i=0; i<n; ++i)
         {
         tp = *(t++);
         bp = *(b++);
         if ( tp > bp )
            p = sp + tp;
         else
            p = sp - bp;
         *(o++) = (p > WHITE) ? WHITE : ((p < BLACK) ? BLACK : p);
         }
      }
   else if ( botname )
      {
      b = B;
      o = O;
      sp = WHITE >> 1;
      for ( i=0; i<n; ++i)
         {
         p = sp - *(b++);
         *(o++) = (p < BLACK) ? BLACK : p;
         }
      }
   else if ( topname )
      {
      t = T;
      o = O;
      sp = WHITE >> 1;
      for ( i=0; i<n; ++i)
         {
         p = *(t++) + sp;
         *(o++) = (p > WHITE) ? WHITE : p;
         }
      }



        
   /* create output file */
   if ( !(fp = OpenFile( outname, "", "w" )) ) exit( 0 );

   if ( !botname )
      WriteRasterFile( fp, &hT, cmT, O, 0, 0, 0, 0 ); 
   else
      WriteRasterFile( fp, &hB, cmB, O, 0, 0, 0, 0 );
                  
   }
