/* extrema:  find the luminance extrema, mean, and std. dev. in a rasfile */
/* version 1.0     2 Oct 1991 */

/* by Richard Alan Peters II                        */
/* Department of Electrical Engineering             */
/* Vanderbilt University School of Engineering      */ 
/* Nashville, TN 37235                              */ 
/* rap2@vuse.vanderbilt.edu                         */ 
/*                                                  */ 
/* This software is freely redistributable if       */ 
/* the author's name and affiliation are included.  */


#include <math.h>		
#include <stdio.h>
#include <strings.h>
#include <values.h>
#include "rasterio.h"
#include "extrema.h"

#define ALLOC 1


main( argc, argv ) 
   unsigned int argc;    /* count of arguments parsed by operating system    */
   char *argv[];         /* pointers to arguments parsed by operating system */
   {
   struct rasterfile H;	/* rasterfile header */
   byte *cmap;			/* colormap */
   byte *I;				/* image pointer */
   byte *u;   			/* image pointer */
   int max,min;			/* extrema */
   int nmax,nmin;		/* number of pixels with extreme value */
   double mean,sdev;	/* some stats */
   unsigned int accum;	/* accumulator */
   double faccum,raccum;
   double x;
   int i,j;				/* indices */


   if (argc >1 )
      {
      fprintf(stderr,
         "usage: extrema <ras_image\n");
      exit(0);
      }


   /* get image from stdin */
   if (  ReadRasterFile( stdin, &H, &cmap, &I, 0, 0, 0, 0, ALLOC )  ) exit( 0 );


   /* get the luminance component */
   ExtractLuminance( &H, cmap, I, 0, 0, 0, 0 );

   accum = 0;
   max = BLACK;
   min = WHITE;
   u = I;

   for (i=0; i<H.ras_length; ++i )
      {
      min = MIN((int)*u,min);
      max = MAX((int)*u,max);
      accum += (int)*(u++);
      }

   mean = (double)accum / (double)H.ras_length;
   faccum = 0.0;
   nmin = 0;
   nmax = 0;
   u = I;

   for (i=0; i<H.ras_height; ++i )
      {
      raccum = 0.0;
      for (j=0; j<H.ras_width; ++j )
         {
         if ( *u == min )
            ++nmin;
         if ( *u == max )
            ++nmax;
         x = (double)*(u++) - mean;
         raccum += x*x ;
         }
      faccum += raccum;
      }

   sdev = sqrt( faccum / (double)H.ras_length );

   fprintf( stderr, "min:  %d   # pixels: %d \n", min, nmin );
   fprintf( stderr, "max:  %d   # pixels: %d \n", max, nmax );
   fprintf( stderr, "mean: %f    std dev: %f \n", mean, sdev);

   }

